/*****************************************************************************
 * AesExamples.c
 *****************************************************************************/

#include <sys/platform.h>
#include "AesExamples.h"
#include <stdio.h>
#include <string.h>
#include <stdbool.h>

#include "data.c"
/* Managed drivers and/or services include */
#include "../Project/SC589/AesExamples_Core0/system/adi_initialize.h"


void aes_dhm_example1(bool use_dec_key);

void aes_dhm_example2(void);

void aes_arm_example1(void);

#if defined(__ADSPSC58x__) && __ADSPSC58x__==1
/* redefine the mmu init function so cache isn't enabled */
void adi_mmu_Init()
{
	asm("nop;");
}
#endif

int main(int argc, char *argv[])
{
	/* Initialize managed drivers and/or services */
	adi_initComponents();


	aes_dhm_example1(false);

	aes_dhm_example1(true);

	aes_dhm_example2();
	
	aes_arm_example1();

	return 0;
}

#if 1
/*
 * This function shows an example of how the PKTE is used in Direct Host Mode (DHM).
 * Everything is register based; configuration and data.
 *
 * The first example data set is from the FIPS-192 AES specification.
 * Input:
 *  plaintext =  00112233445566778899aabbccddeeff
 *  key =        000102030405060708090a0b0c0d0e0f
 *
 * Output:
 *  ciphertext = 69c4e0d86a7b0430d8cdb78070b4c55a
 *
 *
 */
void aes_dhm_example1(bool use_dec_key)
{
	int i, stat, len, resStat;
	uint32_t outputCnt;

	/*
	 * Configure engine
	 */
	*pREG_PKTE0_CFG = 3;         // reset engine and release
	ssync();                     // ensure write propagates through before releasing engine out of reset
	*pREG_PKTE0_CFG = 0;         // release out of reset and put into DH mode


	/*
	 * Write Command Descriptor regs
	 */
	*pREG_PKTE0_CTL_STAT = 0x1;                            // host ready
	*pREG_PKTE0_LEN      = SZ_INPUT1 | 0x400000;           // set length.  Set sync bits PE_READY=0, HOST_READY=1
	*pREG_PKTE0_CDSC_CNT = 1;                              // cmd desc cnt =1.  Trigger PKTE to validate cmd desc

	/*
	 * Configure SA regs to define operation and State regs
	 * to provide any extra input
	 */
	*pREG_PKTE0_SA_CMD0 =
			(((null_hash) << BITP_PKTE_SA_CMD0_HASH)   & BITM_PKTE_SA_CMD0_HASH)   |
			(((aes)       << BITP_PKTE_SA_CMD0_CIPHER) & BITM_PKTE_SA_CMD0_CIPHER) |
			(((zero)      << BITP_PKTE_SA_CMD0_PADTYPE)& BITM_PKTE_SA_CMD0_PADTYPE)|
			(((basic)     << BITP_PKTE_SA_CMD0_OPGRP)  & BITM_PKTE_SA_CMD0_OPGRP)  |
			(((outbound)  << BITP_PKTE_SA_CMD0_DIR)    & BITM_PKTE_SA_CMD0_DIR )   |
			(((encrypt)   << BITP_PKTE_SA_CMD0_OPCD)   & BITM_PKTE_SA_CMD0_OPCD );

	*pREG_PKTE0_SA_CMD1  =
			(((bits_128)  << BITP_PKTE_SA_CMD1_AESKEYLEN) & BITM_PKTE_SA_CMD1_AESKEYLEN) |
			(((ecb)       << BITP_PKTE_SA_CMD1_CIPHERMD)  & BITM_PKTE_SA_CMD1_CIPHERMD) ;


	/*
	 * Program in the key for encryption
	 */
	*pREG_PKTE0_SA_KEY0 = ((uint32_t *)key1)[0];
	*pREG_PKTE0_SA_KEY1 = ((uint32_t *)key1)[1];
	*pREG_PKTE0_SA_KEY2 = ((uint32_t *)key1)[2];
	*pREG_PKTE0_SA_KEY3 = ((uint32_t *)key1)[3];


	/*
	 * trigger operation to begin and have it wait for input
	 */
	*pREG_PKTE0_SA_RDY   = 1;


	/*
	 * write data to input buffer
	 */
	for (i=0; i<SZ_INPUT1/sizeof(uint32_t); i++)
		pREG_PKTE0_DATAIO_BUF[i] = ((uint32_t *)plaintext1)[i];

	/*
	 * Set input buffer count.  round up length to next multiple of 4
	 */
	*pREG_PKTE0_INBUF_CNT = ((SZ_INPUT1+3)&0xfffffffc);

	/*
	 * Poll status for completion
	 */
	stat = *pREG_PKTE0_STAT;
	while((stat & BITM_PKTE_STAT_OPDN) == 0){
		for(i=0; i<100; i++);        // delay before reading stat again
		stat = *pREG_PKTE0_STAT;
	}


	/*
	 * Get how much output available
	 */
	outputCnt = (*pREG_PKTE0_STAT & BITM_PKTE_STAT_OBUFFULLCNT) >> BITP_PKTE_STAT_OBUFFULLCNT;


	/*
	 * Get output from PKTE data buffer
	 */
	for(i=0; i<outputCnt; i++)
		((uint32_t *)output1)[i] = ((uint32_t *)pREG_PKTE0_DATAIO_BUF)[i];


	/*
	 * Acknowledge how many bytes were read from buffer
	 */
	*pREG_PKTE0_OUTBUF_CNT = outputCnt * sizeof(uint32_t);

	/*
	 * Clean up
	 */
	resStat = *pREG_PKTE0_CTL_STAT;
	len     = *pREG_PKTE0_LEN;
	*pREG_PKTE0_RDSC_CNT = 1;                      /* allows engine to accept new cmd desc's */


	/*
	 * Compare result with expected
	 */
	for(resStat=0, i=0; i<SZ_INPUT1; i++)
		if(output1[i] != expected1[i]){
			printf("test1 encryption FAIL! output does not match expected\n");
			resStat = -1;
			break;
		}
	if(resStat == 0)
		printf("test1 encryption PASS!\n");


	/*
	 *
	 * For completeness, decrypt the output and get back the original input
	 *
	 */



	/*
	 * Write Command Descriptor regs
	 */
	*pREG_PKTE0_CTL_STAT = 0x1;                            // host ready
	*pREG_PKTE0_LEN      = SZ_INPUT1 | 0x400000;           // set length.  Set sync bits PE_READY=0, HOST_READY=1
	*pREG_PKTE0_CDSC_CNT = 1;                              // cmd desc cnt =1.  Trigger PKTE to validate cmd desc

	/*
	 * Configure SA regs to define operation and State regs
	 * to provide any extra input.  No need to set SA_CMD1.  Nothing has changed.
	 */
	*pREG_PKTE0_SA_CMD0 =
			(((null_hash) << BITP_PKTE_SA_CMD0_HASH)   & BITM_PKTE_SA_CMD0_HASH)   |
			(((aes)       << BITP_PKTE_SA_CMD0_CIPHER) & BITM_PKTE_SA_CMD0_CIPHER) |
			(((zero)      << BITP_PKTE_SA_CMD0_PADTYPE)& BITM_PKTE_SA_CMD0_PADTYPE)|
			(((basic)     << BITP_PKTE_SA_CMD0_OPGRP)  & BITM_PKTE_SA_CMD0_OPGRP)  |
			(((inbound)   << BITP_PKTE_SA_CMD0_DIR)    & BITM_PKTE_SA_CMD0_DIR )   |
			(((decrypt)   << BITP_PKTE_SA_CMD0_OPCD)   & BITM_PKTE_SA_CMD0_OPCD );

	/*
	 * Use the decryption key (last key from key schedule calculated during encryption process).
	 * If decryption key is used, the packet engine can save cycles by not needing to
	 * recalculate the key schedule to find the last key.
	 */

	if(use_dec_key == true){
		*pREG_PKTE0_SA_CMD1 = *pREG_PKTE0_SA_CMD1 | (1 << BITP_PKTE_SA_CMD1_AESDECKEY);

		*pREG_PKTE0_SA_KEY0 = ((uint32_t *)dec_key1)[0];
		*pREG_PKTE0_SA_KEY1 = ((uint32_t *)dec_key1)[1];
		*pREG_PKTE0_SA_KEY2 = ((uint32_t *)dec_key1)[2];
		*pREG_PKTE0_SA_KEY3 = ((uint32_t *)dec_key1)[3];
	}

	/*
	 * Trigger operation to begin and have it wait for input.  No need to write key again.
	 * It's the same
	 */
	*pREG_PKTE0_SA_RDY   = 1;

	/*
	 * Write data to input buffer.  We can use the data from the output buffer
	 * or the expected buffer but they should be the same.
	 */
	for (i=0; i<SZ_INPUT1/sizeof(uint32_t); i++)
		pREG_PKTE0_DATAIO_BUF[i] = ((uint32_t *)expected1)[i];

	/*
	 * Set input buffer count.  round up length to next multiple of 4
	 */
	*pREG_PKTE0_INBUF_CNT = ((SZ_INPUT1+3)&0xfffffffc);

	/*
	 * Poll status for completion
	 */
	stat = *pREG_PKTE0_STAT;
	while((stat & BITM_PKTE_STAT_OPDN) == 0){
		for(i=0; i<100; i++);        // delay before reading stat again
		stat = *pREG_PKTE0_STAT;
	}


	/*
	 * Get how much output available
	 */
	outputCnt = (*pREG_PKTE0_STAT & BITM_PKTE_STAT_OBUFFULLCNT) >> BITP_PKTE_STAT_OBUFFULLCNT;


	/*
	 * Get output from PKTE data buffer
	 */
	for(i=0; i<outputCnt; i++)
		((uint32_t *)output1)[i] = ((uint32_t *)pREG_PKTE0_DATAIO_BUF)[i];


	/*
	 * Acknowledge how many bytes were read from buffer
	 */
	*pREG_PKTE0_OUTBUF_CNT = outputCnt * sizeof(uint32_t);

	/*
	 * Clean up
	 */
	resStat = *pREG_PKTE0_CTL_STAT;
	len     = *pREG_PKTE0_LEN;
	*pREG_PKTE0_RDSC_CNT = 1;                      /* allows engine to accept new cmd desc's */

	/*
	 * Compare result with expected
	 */
	for(resStat=0, i=0; i<SZ_INPUT1; i++)
		if(output1[i] != plaintext1[i]){
			printf("test1 decryption FAIL! output does not match expected\n");
			resStat = -1;
			break;
		}
	if(resStat == 0)
		printf("test1 decryption PASS!\n");


	return;

}




/*
 * This function shows an example of how the PKTE is used in Direct Host Mode (DHM).
 * Everything is register based; configuration and data.
 *
 * The data set here is from the FIPS CAVP used to validate the PKTE on the BF70x
 * CBCMMT128.  To show how the PKTE is used when the data buffer is filled to maximum capacity,
 * the input is doubled.  The expected output is calculated using openssl.
 *
 * Input:
 *  plaintext =
 *  1e917d7268b847272c3c86b6a506229715579a82ff0c14b12864d593f8da3b622900958c4b0189b3e3288db7d1df6742ac6869b67a0e77a50
 *  00f0fbfb5910b4fa56b29937c8481a32ca97c9aa9433c94eadac2abd32b9f592f77ae598b911403f1f3c71f3141339e38b343a7674ddd5b6e
 *  629ce93caf11b5d306595538549d1bb95193c71483db94d3acfe8fd8d8d8f7c9e3c4007cdcc4c2c252d119b228feea1e917d7268b847272c3
 *  c86b6a506229715579a82ff0c14b12864d593f8da3b622900958c4b0189b3e3288db7d1df6742ac6869b67a0e77a5000f0fbfb5910b4fa56b
 *  29937c8481a32ca97c9aa9433c94eadac2abd32b9f592f77ae598b911403f1f3c71f3141339e38b343a7674ddd5b6e629ce93caf11b5d3065
 *  95538549d1bb95193c71483db94d3acfe8fd8d8d8f7c9e3c4007cdcc4c2c252d119b228feea
 *  key =        9d2880fea346da172a81fa83bb8fcd52
 *  iv =         851c83b61b61dc76ba27e77f5a508d6c
 *
 * Output:
 *  ciphertext =
 *  16d8a57026b0436616a93b18795ad67804f592de422775098940a1612d94297ef32123eec972fe33b4c3160c096586f6e0643df1c6685587b
 *  2d9a00b3b46831f0708077887222de6e16c828e2ef1f47ae695e45598fb7c6fdfa54308915deeca39d47a97c88a778587eda72750c5c70ffe
 *  cfad9022a893c9f8edb3de99a1148971cfa3d2f470851a8d8eac5ed7aaa8ba105cc02ccb39429be4f91718b9da9a6385f6cc6bf6d310581e3
 *  a4ce515798a4b015f743ffc4a43e66ec76dfa78d92219480ab885362250dfa2243f539ca48414f39ac1aec29ca3586a27cfcf663633414d0b
 *  c0a158ad49c9c555b69d710da850492e64c169c7b07dd8d5a1fab136f5ccd58004e1d941d9b1fa71edc6886bc6a84670d9d1b21ec5c153cf5
 *  8e7dbb0f8916437564c1190ff8af38d618d3b8de13765b9dfcb0d179e3a980545f8af59d742
 *
 */
void aes_dhm_example2(void)
{
	int i, stat, len, resStat, offset, remainder;
	uint32_t outputCnt;

	/*
	 * Configure engine
	 */
	*pREG_PKTE0_CFG = 3;         // reset engine and release
	ssync();                     // ensure write propagates through before releasing engine out of reset
	*pREG_PKTE0_CFG = 0;         // release out of reset and put into DH mode


	/*
	 * Write Command Descriptor regs
	 */
	*pREG_PKTE0_CTL_STAT = 0x1;                            // host ready
	*pREG_PKTE0_LEN      = SZ_INPUT2 | 0x400000;           // set length.  Set sync bits PE_READY=0, HOST_READY=1
	*pREG_PKTE0_CDSC_CNT = 1;                              // cmd desc cnt =1.  Trigger PKTE to validate cmd desc

	/*
	 * Configure SA regs to define operation and State regs
	 * to provide any extra input
	 */
	*pREG_PKTE0_SA_CMD0 =
			(((iv_from_state) << BITP_PKTE_SA_CMD0_IVSRC)  & BITM_PKTE_SA_CMD0_IVSRC)  |
			(((null_hash)     << BITP_PKTE_SA_CMD0_HASH)   & BITM_PKTE_SA_CMD0_HASH)   |
			(((aes)           << BITP_PKTE_SA_CMD0_CIPHER) & BITM_PKTE_SA_CMD0_CIPHER) |
			(((zero)          << BITP_PKTE_SA_CMD0_PADTYPE)& BITM_PKTE_SA_CMD0_PADTYPE)|
			(((basic)         << BITP_PKTE_SA_CMD0_OPGRP)  & BITM_PKTE_SA_CMD0_OPGRP)  |
			(((outbound)      << BITP_PKTE_SA_CMD0_DIR)    & BITM_PKTE_SA_CMD0_DIR )   |
			(((encrypt)       << BITP_PKTE_SA_CMD0_OPCD)   & BITM_PKTE_SA_CMD0_OPCD );

	*pREG_PKTE0_SA_CMD1  =
			(((bits_128)  << BITP_PKTE_SA_CMD1_AESKEYLEN) & BITM_PKTE_SA_CMD1_AESKEYLEN) |
			(((cbc)       << BITP_PKTE_SA_CMD1_CIPHERMD)  & BITM_PKTE_SA_CMD1_CIPHERMD) ;


	/*
	 * Program in the key for encryption
	 */
	*pREG_PKTE0_SA_KEY0 = ((uint32_t *)key2)[0];
	*pREG_PKTE0_SA_KEY1 = ((uint32_t *)key2)[1];
	*pREG_PKTE0_SA_KEY2 = ((uint32_t *)key2)[2];
	*pREG_PKTE0_SA_KEY3 = ((uint32_t *)key2)[3];


	/*
	 * Program in the IV
	 */
	*pREG_PKTE0_STATE_IV0 = ((uint32_t *)iv2)[0];
	*pREG_PKTE0_STATE_IV1 = ((uint32_t *)iv2)[1];
	*pREG_PKTE0_STATE_IV2 = ((uint32_t *)iv2)[2];
	*pREG_PKTE0_STATE_IV3 = ((uint32_t *)iv2)[3];

	/*
	 * Trigger operation to begin and have it wait for input.  No need to write key again.
	 * It's the same
	 */
	*pREG_PKTE0_SA_RDY   = 1;



	/*
	 * Provide input in multiple parts and grab output as multiple parts
	 */
	do{

		remainder = SZ_INPUT2;
		offset = 0;
		while(remainder > PKTE_BUFFER_SIZE){

			/*
			 * write in maximum buffer size worth of data
			 */
			for (i=0; i<PKTE_BUFFER_SIZE/sizeof(uint32_t); i++)
				pREG_PKTE0_DATAIO_BUF[i] = ((uint32_t *)plaintext2)[i];

			/*
			 * Set input buffer count.  round up length to next multiple of 4
			 */
			*pREG_PKTE0_INBUF_CNT = PKTE_BUFFER_SIZE;

			/*
			 * Get how much output available
			 */
			while(((*pREG_PKTE0_STAT & BITM_PKTE_STAT_OBUFFULLCNT)>>BITP_PKTE_STAT_OBUFFULLCNT) != PKTE_BUFFER_SIZE/sizeof(uint32_t))
				;
			outputCnt = (*pREG_PKTE0_STAT & BITM_PKTE_STAT_OBUFFULLCNT) >> BITP_PKTE_STAT_OBUFFULLCNT;

			/*
			 * Get output from PKTE data buffer
			 */
			for(i=0; i<outputCnt; i++)
				((uint32_t *)output2)[i+offset] = ((uint32_t *)pREG_PKTE0_DATAIO_BUF)[i];

			/*
			 * Update count of remaining input
			 */
			remainder -= PKTE_BUFFER_SIZE;

			/*
			 * Update offset into output buffer
			 */
			offset += PKTE_BUFFER_SIZE;


			/*
			 * Acknowledge how many bytes were read from buffer
			 */
			*pREG_PKTE0_OUTBUF_CNT = outputCnt * sizeof(uint32_t);

		}

		/*
		 * Take care of rest of data that is of size less than packet buffer size
		 */


		/*
		 * write in rest of data
		 */
		for (i=0; i<remainder/sizeof(uint32_t); i++)
			pREG_PKTE0_DATAIO_BUF[i] = ((uint32_t *)plaintext2)[i+offset/sizeof(uint32_t)];

		/*
		 * Set input buffer count.  round up length to next multiple of 4
		 */
		*pREG_PKTE0_INBUF_CNT = ((remainder + 3) & 0xfffffffc);

		/*
		 * Get how much output available
		 */
		while(((*pREG_PKTE0_STAT & BITM_PKTE_STAT_OBUFFULLCNT)>>BITP_PKTE_STAT_OBUFFULLCNT) != remainder/sizeof(uint32_t))
			;
		outputCnt = (*pREG_PKTE0_STAT & BITM_PKTE_STAT_OBUFFULLCNT) >> BITP_PKTE_STAT_OBUFFULLCNT;

		/*
		 * Get output from PKTE data buffer
		 */
		for(i=0; i<outputCnt; i++)
			((uint32_t *)output2)[i+offset/sizeof(uint32_t)] = ((uint32_t *)pREG_PKTE0_DATAIO_BUF)[i];


		/*
		 * Acknowledge how many bytes were read from buffer
		 */
		*pREG_PKTE0_OUTBUF_CNT = outputCnt * sizeof(uint32_t);


	} while( (*pREG_PKTE0_STAT & BITM_PKTE_STAT_OPDN) == 0);


	/*
	 * Compare result with expected
	 */
	for(resStat=0, i=0; i<SZ_INPUT2; i++)
		if(output2[i] != expected2[i]){
			printf("test2 encryption FAIL! output does not match expected\n");
			resStat = -1;
			break;
		}
	if(resStat == 0)
		printf("test2 encryption PASS!\n");


	return;
}


/*
 * For this example, we will re-do examples 1 and 2 from the Direct Host Mode examples
 * and process them using Autonomous Ring Mode.  Two command descriptors will be set up first
 * and then passed to packet engine.
 *
 * In the 2nd example, the IV will be part of the input buffer instead of being passed in the
 * SA State.  Also, the 2nd example demonstrates how to bypass and ignore data at the beginning
 * of an input buffer.
 */

#define RING_SIZE 2
#define EXAMPLE1_ID 0x1111
#define EXAMPLE2_ID 0x2222

/*
 * Create command descriptor ring and result descriptor ring
 */
section("seg_l2_uncached")
cd_t c_ring[RING_SIZE];
section("seg_l2_uncached")
rd_t r_ring[RING_SIZE];

/*
 * Create the SA records.  We'll have one for each command desc
 */
section("seg_l2_uncached")
sa_t saRec[RING_SIZE];

/*
 * Create the SA State buffers.  We'll have one for each command desc
 */
section("seg_l2_uncached")
state_t saState[RING_SIZE];
void aes_arm_example1(void)
{

	int i, resStat;
	cd_t *cmdDesc;
	sa_t *sR;
	uint8_t *pt;

	/*
	 * set Security on PKTE to be secure master
	 */
#if   defined(__ADSPSC58x__) && __ADSPSC58x__==1
	*pREG_SPU0_SECUREP162 = 3;
#elif defined(__ADSPBF70x__) && __ADSPBF70x__==1
	*pREG_SPU0_SECUREP65  = 3;
#endif

	/*
	 * Clear outputs
	 */
	memset(output1, 0, SZ_INPUT1);
	memset(output2, 0, SZ_INPUT2);


	/*
	 * Configure engine
	 */
	*pREG_PKTE0_CFG = 3;         // reset engine and release
	ssync();                     // ensure write propagates through before releasing engine out of reset
	*pREG_PKTE0_CFG = 0x300;     // release out of reset and put into AR mode


	/*
	 * Configure rings
	 */
	*pREG_PKTE0_RING_CFG = RING_SIZE-1;                // tell engine ring size
	*pREG_PKTE0_CDRBASE_ADDR = (uint32_t)(c_ring);     // tell engine where base of command desc ring is
	*pREG_PKTE0_RDRBASE_ADDR = (uint32_t)(r_ring);     // tell engine where base of result desc ring is


	/*
	 *
	 * Configure descriptor for 1st example
	 *
	 */

	/*
	 * set up command descriptor
	 */
	cmdDesc = &c_ring[0];

	cmdDesc->CTRL_STAT      = BITM_PKTE_CTL_STAT_HOSTRDY;        // transfer ownership from host to PE of the desc
	cmdDesc->SRC_ADDR       = plaintext1;
	cmdDesc->DEST_ADDR      = output1;
	cmdDesc->SA_ADDR        = (unsigned long)(&saRec[0]);
	cmdDesc->SA_STATE_ADDR  = (unsigned long)(&saState[0]);
#if defined(__ADSPSC58x__) && __ADSPSC58x__==1
    cmdDesc->ARC4_STATE_ADDR= 0;
#endif
	cmdDesc->USERID         = EXAMPLE1_ID;
	cmdDesc->LENGTH         = SZ_INPUT1 | BITM_PKTE_LEN_HSTRDY;

	/*
	 *
	 * configure the SA Record
	 *
	 */

	sR = &saRec[0];

	/*
	 * Load in the key. The packet engine will read it from the SA Record
	 */
	sR->KEY_0 = ((unsigned long *)key1)[0];
	sR->KEY_1 = ((unsigned long *)key1)[1];
	sR->KEY_2 = ((unsigned long *)key1)[2];
	sR->KEY_3 = ((unsigned long *)key1)[3];

	/*
	 *  configure SA CMDs to define the job
	 */
	sR->CMD_0 =
			(((null_hash)     << BITP_PKTE_SA_CMD0_HASH)   & BITM_PKTE_SA_CMD0_HASH)   |
			(((aes)           << BITP_PKTE_SA_CMD0_CIPHER) & BITM_PKTE_SA_CMD0_CIPHER) |
			(((zero)          << BITP_PKTE_SA_CMD0_PADTYPE)& BITM_PKTE_SA_CMD0_PADTYPE)|
			(((basic)         << BITP_PKTE_SA_CMD0_OPGRP)  & BITM_PKTE_SA_CMD0_OPGRP)  |
			(((outbound)      << BITP_PKTE_SA_CMD0_DIR)    & BITM_PKTE_SA_CMD0_DIR )   |
			(((encrypt)       << BITP_PKTE_SA_CMD0_OPCD)   & BITM_PKTE_SA_CMD0_OPCD );

	sR->CMD_1 =
			(((bits_128)  << BITP_PKTE_SA_CMD1_AESKEYLEN) & BITM_PKTE_SA_CMD1_AESKEYLEN) |
			(((ecb)       << BITP_PKTE_SA_CMD1_CIPHERMD)  & BITM_PKTE_SA_CMD1_CIPHERMD) ;


	/*
	 *
	 * Configure descriptor for 2nd example
	 *
	 * For this example, we will demonstrate AES CBC encrypt but the IV will be part of
	 * the input buffer instead of configuring it into the SA State
	 */

	/*
	 * Allocate new buffer for input AND iv AND bypass data
	 */
	pt = (uint8_t *)malloc(SZ_INPUT2 + AES_IV_SZ + BYPASS_SZ_WDS*sizeof(uint32_t));
	if(pt == NULL){
		printf("Error! Cannot allocate buffer for ARM example\n");
		exit(1);
	}

	memset(pt, 0xaa, BYPASS_SZ_WDS*sizeof(uint32_t));                            // fill in memory with data to be bypassed
	memcpy(pt+BYPASS_SZ_WDS*sizeof(uint32_t), iv2, AES_IV_SZ);                   // copy over IV
	memcpy(pt+BYPASS_SZ_WDS*sizeof(uint32_t) +AES_IV_SZ, plaintext2, SZ_INPUT2); //copy over plaintext

	/*
	 * set up command descriptor
	 */
	cmdDesc = &c_ring[1];

	cmdDesc->CTRL_STAT      = BITM_PKTE_CTL_STAT_HOSTRDY;        // transfer ownership from host to PE of the desc
	cmdDesc->SRC_ADDR       = pt;
	cmdDesc->DEST_ADDR      = output2;
	cmdDesc->SA_ADDR        = (unsigned long)(&saRec[1]);
	cmdDesc->SA_STATE_ADDR  = (unsigned long)(&saState[1]);
#if defined(__ADSPSC58x__) && __ADSPSC58x__==1
    cmdDesc->ARC4_STATE_ADDR= 0;
#endif
	cmdDesc->USERID         = EXAMPLE2_ID;
	cmdDesc->LENGTH         = (SZ_INPUT2 + AES_IV_SZ + BYPASS_SZ_WDS * sizeof(uint32_t)) |
			                 ((BYPASS_SZ_WDS << BITP_PKTE_LEN_BYPASS) & BITM_PKTE_LEN_BYPASS) |
			                 BITM_PKTE_LEN_HSTRDY;

	/*
	 *
	 * configure the SA Record
	 *
	 */

	sR = &saRec[1];

	/*
	 * Load in the key. The packet engine will read it from the SA Record
	 */
	sR->KEY_0 = ((unsigned long *)key2)[0];
	sR->KEY_1 = ((unsigned long *)key2)[1];
	sR->KEY_2 = ((unsigned long *)key2)[2];
	sR->KEY_3 = ((unsigned long *)key2)[3];

	/*
	 *  configure SA CMDs to define the job
	 */
	sR->CMD_0 =
			(((iv_from_inbuf) << BITP_PKTE_SA_CMD0_IVSRC)  & BITM_PKTE_SA_CMD0_IVSRC)  |
			(((null_hash)     << BITP_PKTE_SA_CMD0_HASH)   & BITM_PKTE_SA_CMD0_HASH)   |
			(((aes)           << BITP_PKTE_SA_CMD0_CIPHER) & BITM_PKTE_SA_CMD0_CIPHER) |
			(((zero)          << BITP_PKTE_SA_CMD0_PADTYPE)& BITM_PKTE_SA_CMD0_PADTYPE)|
			(((basic)         << BITP_PKTE_SA_CMD0_OPGRP)  & BITM_PKTE_SA_CMD0_OPGRP)  |
			(((outbound)      << BITP_PKTE_SA_CMD0_DIR)    & BITM_PKTE_SA_CMD0_DIR )   |
			(((encrypt)       << BITP_PKTE_SA_CMD0_OPCD)   & BITM_PKTE_SA_CMD0_OPCD );

	sR->CMD_1  =
			(((bits_128)  << BITP_PKTE_SA_CMD1_AESKEYLEN) & BITM_PKTE_SA_CMD1_AESKEYLEN) |
			(((cbc)       << BITP_PKTE_SA_CMD1_CIPHERMD)  & BITM_PKTE_SA_CMD1_CIPHERMD) ;

	/*
	 * Start processing
	 * Increment the number of command descriptors in the ring
	 */
	*pREG_PKTE0_CDSC_INCR = 2;

	/*
	 * Wait until the result descriptor count is also 2.  This means that both command descriptors have
	 * been processed
	 */
	while(*pREG_PKTE0_RDSC_CNT != 2)
		;

	/*
	 * Assume no errors. Compare results.
	 */
	for(resStat=0, i=0; i<SZ_INPUT1; i++)
		if(output1[i] != expected1[i]){
			printf("AR Mode test1 encryption FAIL! output does not match expected\n");
			resStat = -1;
			break;
		}
	if(resStat == 0)
		printf("AR Mode test1 encryption PASS!\n");

	resStat = 0;
	for(i=0; i<BYPASS_SZ_WDS * sizeof(uint32_t); i++)
		if(output2[i] != pt[i]){
			printf("AR Mode test2 encryption FAIL!  Bypass words do not match\n");
			resStat = -1;
			break;
		}
	for(i=0; i<SZ_INPUT2; i++)
		if(output2[i+BYPASS_SZ_WDS * sizeof(uint32_t)] != expected2[i]){
			printf("AR Mode test2 encryption FAIL! output does not match expected\n");
			resStat = -1;
			break;
		}
	if(resStat == 0)
		printf("AR Mode test2 encryption PASS!\n");


	/*
	 * Another check.  The USERIDs from the command descriptor should be passed to the result descriptor
	 */
	if(r_ring[0].USERID != EXAMPLE1_ID)
		printf("AR Mode test error:  UserID 1 doesn't match\n");

	if(r_ring[1].USERID != EXAMPLE2_ID)
		printf("AR Mode test error:  UserID 2 doesn't match\n");



	/*
	 * Acknowledge that the two result descriptors was serviced
	 */
	*pREG_PKTE0_RDSC_DECR = 2;




	/*
	 * Clean up
	 */
	if(pt != NULL)
		free(pt);


	return;
}

#endif
